@@ -2,21 +2,21 @@ import StringIO
 import argparse
 import sys
 import version
+from hepdata_validator import LATEST_SCHEMA_VERSION
 from parsers import Parser
 from writers import Writer
 
-
 def convert(input, output=None, options={}):
-    """Converts arbitrary supported data format to the new HEPData YAML format, and writes the output files
-    to the output_dir directory
+    """Converts a supported ``input_format`` (*oldhepdata*, *yaml*)
+    to a supported ``output_format`` (*csv*, *root*, *yaml*, *yoda*).
 
-    :param input: input, depending on the choosen input datatype it may be, filepath, filetype object, directory, etc
-    :param output: output directory to which converted YAML files will be written
-    :param input_format: format of the input data, must be a string containing name of the input parser class
-    :type input_format: str
-    :param options: additional options used for conversion (depends on the choosen input format)
+    :param input: location of input file for *oldhepdata* format or input directory for *yaml* format
+    :param output: location of output directory to which converted files will be written
+    :param options: additional options such as ``input_format`` and ``output_format`` used for conversion
+    :type input: str
+    :type output: str
     :type options: dict
-    :raise ValueError: raised if no input_format is specified
+    :raise ValueError: raised if no ``input_format`` or ``output_format`` is specified
     """
 
     if 'input_format' not in options and 'output_format' not in options:
@@ -29,7 +29,7 @@ def convert(input, output=None, options={}):
     writer = Writer.get_concrete_class(output_format)(**options)
 
     if not output and not writer.single_file_output:
-        raise ValueError("this output_format requires specifing 'output' argument")
+        raise ValueError("this output_format requires specifying 'output' argument")
 
     # if no output was specified create proxy output to which writer can insert data
     _output = output
@@ -78,6 +78,8 @@ def _main(arguments=sys.argv):
     parser.add_argument("--input-format", '-i', action='store', default='yaml', help='format of the input file/s (default: yaml) [choose one option from Parsers section below]')
     parser.add_argument("--output-format", '-o', action='store', default='yaml', help='format of the output file/s (default: yaml) [choose one option from Writers section below]')
     parser.add_argument("--version", '-v', action='store_const', const=True, default=False, help='Show hepdata-converter version')
+    parser.add_argument("--hepdata-doi", '-d', action='store', default='', help='Pass HEPData DOI, e.g. "10.17182/hepdata.74247.v1"')
+    parser.add_argument("--validator-schema-version", '-s', action='store', default=LATEST_SCHEMA_VERSION, help='hepdata_validator schema version (default: %s)' % LATEST_SCHEMA_VERSION)
     parser.add_argument("input")
     parser.add_argument("output")
 
