@@ -1,7 +1,6 @@
 from string import lower
 from hepdata_converter.common import OptionInitMixin, Option
 from hepdata_converter.parsers import Parser, ParsedData, BadFormat, Table
-import StringIO
 import copy
 import re
 
@@ -19,7 +18,7 @@ class HEPTable(Table):
 class OldHEPData(Parser):
     """Parser for Old HEPData format
     """
-    help = 'Parses OLDHepData format - example OLD HepData input format: http://hepdata.cedar.ac.uk/resource/sample.input'
+    help = 'Parses OLD HepData format - example OLD HepData input format: http://hepdata.cedar.ac.uk/resource/sample.input'
 
     @classmethod
     def options(cls):
@@ -42,6 +41,7 @@ class OldHEPData(Parser):
         self.lines = []
         self.set_state('document')
         self.current_file = None
+        self.set_of_energies = set()
 
     def set_state(self, state):
         if state not in OldHEPData.states:
@@ -60,7 +60,7 @@ class OldHEPData(Parser):
         """
 
 
-        # mapping of OLDHEP format's keywords to proper parsing functions
+        # mapping of OLD HepData format's keywords to proper parsing functions
         # all possible keywords are specified here, the ones which aren't used
         # by new data format are either mapped to _pass method which does nothing,
         # or bound to _add_to_comment which adds the specified data to comment section of the
@@ -74,12 +74,12 @@ class OldHEPData(Parser):
                 'dataset':    self._set_table,
 
                 # additional data which have no one to one mapping to new YAML format
-                'author':     self._bind_parse_addtional_data('author'),
-                'doi':        self._bind_parse_addtional_data('doi'),
-                'status':     self._bind_parse_addtional_data('status'),
-                'experiment': self._bind_parse_addtional_data('experiment'),
-                'detector':   self._bind_parse_addtional_data('detector'),
-                'title':      self._bind_parse_addtional_data('title'),
+                'author':     self._bind_parse_additional_data('author'),
+                'doi':        self._bind_parse_additional_data('doi'),
+                'status':     self._bind_parse_additional_data('status'),
+                'experiment': self._bind_parse_additional_data('experiment'),
+                'detector':   self._bind_parse_additional_data('detector'),
+                'title':      self._bind_parse_additional_data('title'),
 
                 # add it to record_ids
                 'spiresId':   self._bind_parse_record_ids('spires'),
@@ -94,12 +94,14 @@ class OldHEPData(Parser):
                 'dataend':    self._set_document,
                 'location':   self._bind_set_table_metadata('location'),
                 'dscomment':  self._bind_set_table_metadata('description', True),
+                'dserror':    self._parse_dserror,
                 'reackey':    self._parse_reackey,
                 'qual':       self._parse_qual,
                 'data':       self._parse_table_data,
                 'xheader':    self._parse_xheaders,
                 'yheader':    self._parse_yheaders,
                 'obskey':     self._parse_obskey,
+                'phrase':     self._parse_phrase,
                 'E':          self._pass
             }
         }
@@ -112,7 +114,7 @@ class OldHEPData(Parser):
             pass
 
         if self.use_additional_data:
-            if len(self.additional_data) > 0:
+            if self.additional_data:
                 self.data[0]['comment'] += 'ADDITIONAL DATA IMPORTED FROM OLD HEPDATA FORMAT: \n'
             for key in self.additional_data:
                 for element in self.additional_data[key]:
@@ -124,7 +126,7 @@ class OldHEPData(Parser):
         # clean any possible data from previous parsing
         self.reset()
         # in case of strings we should treat them as filepaths
-        if isinstance(data_in, str) or isinstance(data_in, unicode):
+        if isinstance(data_in, (str, unicode)):
             with open(data_in, 'r') as self.current_file:
                 return self._parse()
         else:
@@ -144,11 +146,11 @@ class OldHEPData(Parser):
             return False
 
         # line was empty or it was a comment, continue
-        if line == '\n':
+        if line.strip() == '':
             return True
 
         # retrieve keyword and its value
-        reg = re.search("^\*(?P<key>[^:#]*)(:(?P<value>.*))?$", line)
+        reg = re.search("^\*(?P<key>[^:#]*)(:\s*(?P<value>.*)\s*)?$", line)
         if reg:
             key = reg.group('key').strip()
             value = reg.group('value')
@@ -171,10 +173,12 @@ class OldHEPData(Parser):
         if 'additional_resources' not in self.data[0]:
             self.data[0]['additional_resources'] = []
 
-        self.data[0]['additional_resources'].append({
-            'location': data.split(' : ')[0],
-            'description': ''
-        })
+        location = data.split(' : ')[0].strip()
+        if location.startswith('http'):
+            self.data[0]['additional_resources'].append({
+                'location': location,
+                'description': 'web page with auxiliary material'
+            })
 
     def _set_table(self, data):
         """Set current parsing state to 'table',
@@ -207,18 +211,24 @@ class OldHEPData(Parser):
 
         self.current_table.data_header = header
 
-        for i in range(len(header)):
-            header[i] = header[i].strip()
+        for i, h in enumerate(header):
+            header[i] = h.strip()
 
         x_count = header.count('x')
         y_count = header.count('y')
 
+        if not self.current_table.xheaders:
+            raise BadFormat("*xheader line needs to appear before *data: %s" % data)
+
+        if not self.current_table.yheaders:
+            raise BadFormat("*yheader line needs to appear before *data: %s" % data)
+
         # use deepcopy to avoid references in yaml... may not be required, and will very probably be refactored
-        # TODO - is this aproperiate behavior, or are references in YAML files acceptable, they are certainly less human readable
+        # TODO - is this appropriate behavior, or are references in YAML files acceptable, they are certainly less human readable
         self.current_table.data = {'independent_variables': [{'header': self.current_table.xheaders[i] if i < len(self.current_table.xheaders) else copy.deepcopy(self.current_table.xheaders[-1]),
                                                               'values': []} for i in range(x_count)],
                                       'dependent_variables': [{'header': self.current_table.yheaders[i] if i < len(self.current_table.yheaders) else copy.deepcopy(self.current_table.yheaders[-1]),
-                                                               'qualifiers': [self.current_table.qualifiers[j][i] for j in range(len(self.current_table.qualifiers)) ],
+                                                               'qualifiers': [self.current_table.qualifiers[j][i] if i < len(self.current_table.qualifiers[j]) else copy.deepcopy(self.current_table.qualifiers[j][-1]) for j in range(len(self.current_table.qualifiers)) ],
                                                                'values': []} for i in range(y_count)]}
 
         xy_mapping = []
@@ -226,11 +236,11 @@ class OldHEPData(Parser):
         current_x_count = 0
         current_y_count = 0
 
-        for i in range(len(header)):
-            if header[i] == 'x':
+        for h in header:
+            if h == 'x':
                 xy_mapping.append(current_x_count)
                 current_x_count += 1
-            if header[i] == 'y':
+            if h == 'y':
                 xy_mapping.append(current_y_count)
                 current_y_count += 1
 
@@ -242,66 +252,218 @@ class OldHEPData(Parser):
 
             if len(data_entry_elements) == len(header):
             # this is kind of a big stretch... I assume that x is always first
-                for i in range(len(header)):
-                    single_element = data_entry_elements[i]
+                for i, h in enumerate(header):
+                    single_element = data_entry_elements[i].strip()
 
-                    if header[i] == 'x': # independent variables
-                        r = re.search('(?P<low>[0-9]+\.?[0-9]*)( +TO +(?P<high>[0-9]+\.?[0-9]*))?', single_element)
-                        if r:
-                            if r.group('high') is not None:
-                                single_element = {'low': float(r.group('low')), 'high': float(r.group('high'))}
+                    # number patterns copied from old subs.pl parsing script
+                    pmnum1 = '[-+]?[\d]+\.?[\d]*'
+                    pmnum2 = '[-+]?\.[\d]+'
+                    pmnum3 = '[-+]?[\d]+\.?[\d]*\s*[eE]+\s*[+-]?\s*[\d]+'
+                    pmnum = '(' + pmnum1 + '|' + pmnum2 + '|' + pmnum3 + ')'
+
+                    # implement same regular expression matching as in old subs.pl parsing script
+
+                    if h == 'x': # independent variables
+
+                        r = re.search('^(?P<value>' + pmnum + ')$', single_element)
+                        if r: # "value"
+                            single_element = {'value': r.group('value')}
+                        else:
+                            r = re.search('^(?P<value>' + pmnum + ')\s*\(\s*BIN\s*=\s*(?P<low>' + pmnum + \
+                                          ')\s+TO\s+(?P<high>' + pmnum + ')\s*\)$', single_element)
+                            if r: # "value (BIN=low TO high)"
+                                single_element = {'value': float(r.group('value')),
+                                                  'low': float(r.group('low')), 'high': float(r.group('high'))}
                             else:
-                                single_element = {'value': float(r.group('low'))}
-                            self.current_table.data['independent_variables'][xy_mapping[i]]['values'].append(single_element)
+                                r = re.search('^(?P<low>' + pmnum + ')\s+TO\s+(?P<high>' + pmnum + ')$',
+                                              single_element)
+                                if r: # "low TO high"
+                                    single_element = {'low': float(r.group('low')), 'high': float(r.group('high'))}
+                                else: # everything else: don't try to convert to float
+                                    single_element = {'value': single_element}
 
-                    elif header[i] == 'y': # independant variable
-                        r = re.search('( *)(?P<value>[0-9]+\.?[0-9]*)( *)\+(?P<err_p>[0-9]+\.?[0-9]*),(?P<err_m>\-[0-9]+\.?[0-9]*)( *)\((?P<err_sys>[^()]*)\)', single_element)
+                        # TO DO: subs.pl also parses other formats such as "low high", "value low high" (sorted),
+                        # "value +- err", and "value -err_m, +err_p".  Do we need to support these formats here?
+                        # Probably not: unsupported formats will just be written as a text string.
+
+                        self.current_table.data['independent_variables'][xy_mapping[i]]['values'].append(single_element)
+
+                        # extract energy if SQRT(S) is one of the 'x' variables
+                        xheader = self.current_table.data['independent_variables'][xy_mapping[i]]['header']
+                        if xheader['name'].startswith('SQRT(S)') and lower(xheader['units']) in ('gev'):
+                            for energy in single_element.values():
+                                try:
+                                    energy = float(energy)
+                                    self.set_of_energies.add(energy)
+                                except:
+                                    pass
+
+                    elif h == 'y': # dependent variable
+
+                        pmnum_pct = pmnum + '(\s*PCT)?' # errors can possibly be given as percentages
+
+                        r = re.search('^(?P<value>' + pmnum + ')\s+(?P<err_p>' + pmnum_pct + '|-)\s*,\s*(?P<err_m>' +
+                                      pmnum_pct + '|-)\s*(?P<err_sys>\(\s*DSYS=[^()]+\s*\))?$', single_element)
                         element = {'errors': []}
-                        if r:
-                            element['value'] = float(r.group('value'))
-                            element['errors'] += [{'label': 'stat', 'asymerror': {'plus': float(r.group('err_p')), 'minus': float(r.group('err_m'))}}]
+                        if r: # asymmetric first error
+                            element['value'] = r.group('value').strip()
+                            err_p = r.group('err_p').strip().lstrip('+')
+                            if err_p == '-': err_p = '' # represent missing error as '-' in oldhepdata format
+                            err_p = err_p[:-3].strip() + '%' if err_p[-3:] == 'PCT' else err_p
+                            err_m = r.group('err_m').strip().lstrip('+')
+                            if err_m == '-': err_m = '' # represent missing error as '-' in oldhepdata format
+                            err_m = err_m[:-3].strip() + '%' if err_m[-3:] == 'PCT' else err_m
+                            if err_p and err_m and err_p[-1] != '%' and err_m[-1] == '%':
+                                err_p = err_p + '%'
+                            if not err_p and not err_m:
+                                raise ValueError("Both asymmetric errors cannot be '-': %s" % line)
+                            if r.group('err_sys'):
+                                element['errors'] += [{'label': 'stat', 'asymerror': {'plus': err_p, 'minus': err_m}}]
+                            else:
+                                element['errors'] += [{'asymerror': {'plus': err_p, 'minus': err_m}}]
 
                         else:
-                            r = re.search('( *)(?P<value>[0-9]+\.?[0-9]*)( *)\+-( *)(?P<error>[0-9]+\.?[0-9]*)( *)\((?P<err_sys>[^()]*)\)', single_element)
-                            if r:
-                                element['value'] = float(r.group('value'))
-                                element['errors'] += [{'label': 'stat', 'symerror': float(r.group('error'))}]
+                            r = re.search('^(?P<value>' + pmnum + ')\s*(\+-\s*(?P<error>' +
+                                          pmnum_pct + '))?\s*(?P<err_sys>\(\s*DSYS=[^()]+\s*\))?$', single_element)
+                            if r: # symmetric first error
+                                element['value'] = r.group('value').strip()
+                                if r.group('error'):
+                                    error = r.group('error').strip().lstrip('+')
+                                    error = error[:-3].strip() + '%' if error[-3:] == 'PCT' else error
+                                    if r.group('err_sys'):
+                                        element['errors'] += [{'label': 'stat', 'symerror': error}]
+                                    else:
+                                        element['errors'] += [{'symerror': error}]
+                            else: # everything else
+                                element['value'] = single_element
 
-                        err_sys = r.group('err_sys').split('DSYS=')
+                        err_sys = []
+                        if r and r.group('err_sys'):
+                            err_sys = r.group('err_sys').strip(' \t()').split('DSYS=')
 
-                        for err in err_sys:
+                        for err in err_sys + self.current_table.dserrors:
+                            err = err.strip(' \t,')
                             if not err:
                                 continue
-                            err = err.strip(',')
                             error = {}
                             label = 'sys'
-                            r = re.search('^(?P<error>[0-9]+\.?[0-9]*)(\:(?P<label>.*))?,?$', err)
-                            if r:
+                            r = re.search('^(\+-)?\s*(?P<error>' + pmnum_pct + ')\s*(\:\s*(?P<label>.+))?$', err)
+                            if r: # symmetric systematic error
                                 if r.group('label'):
                                     label += ',' + r.group('label')
-                                error = {'symerror': float(r.group('error')) }
+                                error = r.group('error').strip().lstrip('+')
+                                error = error[:-3].strip() + '%' if error[-3:] == 'PCT' else error
+                                error = {'symerror': error}
 
                             else:
-                                #TODO - check some assumptions - will '+' always be first? and '-' always second?
-                                r = re.search('\+(?P<err_p>[0-9]+\.?[0-9]*)\,(?P<err_m>\-[0-9]+\.?[0-9]*)(\:(?P<label>.*))?', err)
-                                if r:
+                                r = re.search('^(?P<err_p>' + pmnum_pct + '|-)\s*,\s*(?P<err_m>' +
+                                              pmnum_pct + '|-)\s*(\:\s*(?P<label>.+))?$', err)
+                                if r: # asymmetric systematic error
                                     if r.group('label'):
                                         label += ',' + r.group('label')
-                                    error = {'asymerror': {'plus': float(r.group('err_p')), 'minus': float(r.group('err_m'))}}
+                                    err_p = r.group('err_p').strip().lstrip('+')
+                                    if err_p == '-': err_p = '' # represent missing error as '-' in oldhepdata format
+                                    err_p = err_p[:-3].strip() + '%' if err_p[-3:] == 'PCT' else err_p
+                                    err_m = r.group('err_m').strip().lstrip('+')
+                                    if err_m == '-': err_m = '' # represent missing error as '-' in oldhepdata format
+                                    err_m = err_m[:-3].strip() + '%' if err_m[-3:] == 'PCT' else err_m
+                                    if err_p and err_m and err_p[-1] != '%' and err_m[-1] == '%':
+                                        err_p = err_p + '%'
+                                    if not err_p and not err_m:
+                                        raise ValueError("Both asymmetric errors cannot be '-': %s" % line)
+                                    error = {'asymerror': {'plus': err_p, 'minus': err_m}}
                             if not r:
                                 # error happened
-                                raise ValueError("Error while parsing data")
+                                raise ValueError("Error while parsing data line: %s" % line)
 
                             error['label'] = label
-                            element['errors'].append(error)
+                            if element['value'] != single_element:
+                                element['errors'].append(error)
                         self.current_table.data['dependent_variables'][xy_mapping[i]]['values'].append(element)
 
+            elif data_entry_elements:
+                raise BadFormat("%s data entry elements but %s expected: %s" %
+                                (len(data_entry_elements), len(header), line))
+
             last_index = self.current_file.tell()
             l = self.current_file.readline()
             line = self._strip_comments(l)
 
         self.current_file.seek(last_index)
 
+        # extract minimum and maximum from set of energies
+        if self.set_of_energies:
+            energy_min = min(self.set_of_energies)
+            energy_max = max(self.set_of_energies)
+            if energy_max > energy_min:
+                energy = str(energy_min) + '-' + str(energy_max)
+            else:
+                energy = energy_min
+            self._parse_energies(energy)
+
+        if self.current_table.description:
+            if any(word in self.current_table.description.lower() for word in ['covariance', 'correlation', 'matrix']):
+                reformatted = self._reformat_matrix()
+
+    def _reformat_matrix(self):
+        """Transform a square matrix into a format with two independent variables and one dependent variable.
+        """
+        nxax = len(self.current_table.data['independent_variables'])
+        nyax = len(self.current_table.data['dependent_variables'])
+        npts = len(self.current_table.data['dependent_variables'][0]['values'])
+
+        # check if 1 x-axis, and npts (>=2) equals number of y-axes
+        if nxax != 1 or nyax != npts or npts < 2:
+            return False
+
+        # add second independent variable with each value duplicated npts times
+        if len(self.current_table.xheaders) == 2:
+            xheader = self.current_table.xheaders[1]
+        else:
+            xheader = copy.deepcopy(self.current_table.data['independent_variables'][0]['header'])
+        self.current_table.data['independent_variables'].append({'header': xheader, 'values': []})
+        for value in self.current_table.data['independent_variables'][0]['values']:
+            self.current_table.data['independent_variables'][1]['values'].extend([copy.deepcopy(value) for npt in range(npts)])
+
+        # duplicate values of first independent variable npts times
+        self.current_table.data['independent_variables'][0]['values'] \
+            = [copy.deepcopy(value) for npt in range(npts) for value in self.current_table.data['independent_variables'][0]['values']]
+
+        # suppress header if different for second y-axis
+        if self.current_table.data['dependent_variables'][0]['header'] != \
+                self.current_table.data['dependent_variables'][1]['header']:
+            self.current_table.data['dependent_variables'][0]['header'] = {'name': ''}
+
+        # remove qualifier if different for second y-axis
+        iqdel = [] # list of qualifier indices to be deleted
+        for iq, qualifier in enumerate(self.current_table.data['dependent_variables'][0]['qualifiers']):
+            if qualifier != self.current_table.data['dependent_variables'][1]['qualifiers'][iq]:
+                iqdel.append(iq)
+        for iq in iqdel[::-1]: # need to delete in reverse order
+            del self.current_table.data['dependent_variables'][0]['qualifiers'][iq]
+
+        # append values of second and subsequent y-axes to first dependent variable
+        for iy in range(1, nyax):
+            for value in self.current_table.data['dependent_variables'][iy]['values']:
+                self.current_table.data['dependent_variables'][0]['values'].append(value)
+
+        # finally, delete the second and subsequent y-axes in reverse order
+        for iy in range(nyax-1, 0, -1):
+            del self.current_table.data['dependent_variables'][iy]
+
+        return True
+
+    def _parse_dserror(self, data):
+        """Parse dserror attribute of the old HEPData format
+
+        example:
+        *dserror: 7.5 PCT : overall normalization uncertainty
+
+        :param data: data to be parsed
+        :type data: str
+        """
+        self.current_table.dserrors.append(data.strip())
+
     def _parse_reackey(self, data):
         """Parse reackey attribute of the old HEPData format
 
@@ -311,7 +473,7 @@ class OldHEPData(Parser):
         :param data: data to be parsed
         :type data: str
         """
-        self.current_table.reactions.append(data)
+        self.current_table.reactions.append(data.strip())
 
     def _parse_obskey(self, data):
         """Parse obskey attribute of the old HEPData format
@@ -322,11 +484,22 @@ class OldHEPData(Parser):
         :param data: data to be parsed
         :type data: str
         """
-        self.current_table.observables.append(data)
+        self.current_table.observables.append(data.strip())
+
+    def _parse_phrase(self, data):
+        """Parse phrase attribute of the old HEPData format
+
+        example:
+        *phrase: Z pair Production
+
+        :param data: data to be parsed
+        :type data: str
+        """
+        self.current_table.phrases.append(data.strip())
 
     def _parse_energies(self, data):
         """Add energy given in data to tables energies
-        this method is here for compleatness sake, it's used in only one other place so
+        this method is here for completeness sake, it's used in only one other place so
         can be safely extracted
 
         :param data: data to be appended to table's energies
@@ -347,12 +520,15 @@ class OldHEPData(Parser):
         headers = data.split(':')
         name = headers[0].strip()
 
-        name = name.split(' IN ')
+        name = re.split(' IN ', name, flags=re.I) # ignore case
         units = None
         if len(name) > 1:
             units = name[1].strip()
         name = name[0].strip()
 
+        if len(headers) < 2:
+            raise BadFormat("*qual line must contain a name and values: %s" % data)
+
         for header in headers[1:]:
             xheader = {'name': name}
             if units:
@@ -361,11 +537,15 @@ class OldHEPData(Parser):
             xheader['value'] = header.strip()
             list.append(xheader)
 
-            #:TODO: VERY RISKY, check energies and so on
-            #this is a hack, I hope there is better way to do this:
-            if name == 'SQRT(S)' and lower(units) in ('gev', 'tev', 'mev'):
-                if xheader['value'] not in self.current_table.energies:
-                    self._parse_energies(xheader['value'])
+            # extract energy if SQRT(S) is one of the qualifiers
+            if name.startswith('SQRT(S)') and lower(units) in ('gev'):
+                energies = re.split(' TO ', xheader['value'], flags=re.I)
+                for energy in energies:
+                    try:
+                        energy = float(energy)
+                        self.set_of_energies.add(energy)
+                    except:
+                        pass
 
         self.current_table.qualifiers.append(list)
 
@@ -382,7 +562,7 @@ class OldHEPData(Parser):
         headers = data.split(':')
 
         for header in headers:
-            header = header.split(' IN ')
+            header = re.split(' IN ', header, flags=re.I) # ignore case
             xheader = {'name': header[0].strip()}
             if len(header) > 1:
                 xheader['units'] = header[1].strip()
@@ -434,6 +614,7 @@ class OldHEPData(Parser):
         """
         result = init_data
 
+        first = True
         while True:
             last_index = self.current_file.tell()
             line_raw = self.current_file.readline()
@@ -443,7 +624,7 @@ class OldHEPData(Parser):
                 continue
 
             # now strip comments
-            # TODO - is it aproperiate behavior?
+            # TODO - is it appropriate behavior?
             data = self._strip_comments(line_raw)
             # EOF, stop here
             if not data:
@@ -454,7 +635,16 @@ class OldHEPData(Parser):
                 self.current_file.seek(last_index)
                 break
 
+            if first:
+                result += '\n'
+                first = False
+
             result += data
+
+        result = result.strip()
+        if result and not result.endswith('.'):
+            result += '.'
+
         return result
 
     def _parse_document_comment(self, data):
@@ -474,7 +664,9 @@ class OldHEPData(Parser):
         def set_table_metadata(self, data):
             if multiline:
                 data = self._read_multiline(data)
-            self.current_table.metadata[key] = data
+            if key == 'location' and data:
+                data = 'Data from ' + data
+            self.current_table.metadata[key] = data.strip()
 
         # method must be bound, so we use __get__
         return set_table_metadata.__get__(self)
@@ -484,7 +676,7 @@ class OldHEPData(Parser):
             if 'record_ids' not in self.data[0]:
                 self.data[0]['record_ids'] = []
 
-            record_id = {'type': key, 'id': int(data)}
+            record_id = {'type': key, 'id': int(data) if data else 0}
 
             if self.data[0]['record_ids'].count(record_id) == 0:
                 self.data[0]['record_ids'].append(record_id)
@@ -494,7 +686,7 @@ class OldHEPData(Parser):
         # method must be bound, so we use __get__
         return _parse_record_ids.__get__(self)
 
-    def _bind_parse_addtional_data(self, key, multiline=False):
+    def _bind_parse_additional_data(self, key, multiline=False):
         """Returns parsing function which will parse data as text, and add it to the table additional data dictionary
         with the provided key
 
