@@ -1,12 +1,10 @@
 import inspect
 import os
 import pkgutil
-import shutil
 from hepdata_converter.common import GetConcreteSubclassMixin, OptionInitMixin
 
 __all__ = []
 
-from string import lower
 import abc
 
 
@@ -16,6 +14,7 @@ class Writer(GetConcreteSubclassMixin, OptionInitMixin):
     def __init__(self, single_file_output, *args, **kwargs):
         OptionInitMixin.__init__(self, options=kwargs)
         self.single_file_output = single_file_output
+        self.hepdata_doi = kwargs['hepdata_doi'].encode('utf8', 'replace') if 'hepdata_doi' in kwargs else ''
 
     @classmethod
     def create_dir(cls, path):
@@ -27,7 +26,7 @@ class Writer(GetConcreteSubclassMixin, OptionInitMixin):
 
     @abc.abstractmethod
     def write(self, data_in, data_out, *args, **kwargs):
-        """Writes data provided by data_in argument to data_out, data_out is implementation dependant, and can be
+        """Writes data provided by data_in argument to data_out, data_out is implementation dependent, and can be
         directory, filelike object, etc
 
         :param data_in: input data (type is implementation specified)
@@ -37,7 +36,7 @@ class Writer(GetConcreteSubclassMixin, OptionInitMixin):
         """
 
 
-# import all packages in the parsers package, so that Parser.get_specific_parser will recognise them
+# import all packages in the parsers package, so that Writer.get_specific_writer will recognise them
 for loader, name, is_pkg in pkgutil.walk_packages(__path__):
     module = loader.find_module(name).load_module(name)
 
