@@ -32,20 +32,27 @@ class THFRootClass(ObjectWrapper):
             for independent_variable in independent_variables_map:
                 if not independent_variable['values']:
                     return False
-                if 'low' not in independent_variable['values'][0] or 'high' not in independent_variable['values'][0]:
-                    return False
+                for value in independent_variable['values']:
+                    if 'low' not in value or 'high' not in value:
+                        return False
             return True
         return False
 
     def _create_empty_hist(self, dependent_var_title, index, yval):
 
+        is_number_list = self.is_number_var(self.dependent_variable)
+
         xval = []
         for i in xrange(self.dim):
             xval.append([])
             i_var = self.independent_variables[i]['values']
-            for x in i_var:
-                xval[i].append(x['low'])
-            xval[i].append(i_var[-1]['high'])
+            for ix, x in enumerate(i_var):
+                if not is_number_list[ix] and 'labels' not in self.independent_variables[i]:
+                    continue  # skip defining bins for non-numeric y values unless alphanumeric bin labels are present
+                if x['low'] not in xval[i]:
+                    xval[i].append(x['low'])
+                if x['high'] not in xval[i]:
+                    xval[i].append(x['high'])
 
         name = "Hist%sD_y%s_e%s" % (self.dim, self.dependent_variable_index + 1, index)
 
@@ -53,14 +60,7 @@ class THFRootClass(ObjectWrapper):
         xval_ordered = []
         for i in xrange(self.dim):
             xval_ordered.append([])
-            for j, x in enumerate(xval[i]):
-                if j == 0:
-                    x_highest = x
-                    xval_ordered[i].append(x)
-                else:
-                    if x > x_highest:
-                        x_highest = x
-                        xval_ordered[i].append(x)
+            xval_ordered[i] = sorted(xval[i])
 
         if 1 == self.dim:
             nbinsx = len(xval_ordered[0]) - 1
@@ -87,6 +87,7 @@ class THFRootClass(ObjectWrapper):
             name = self.independent_variables[i]['header']['name']
             if 'units' in self.independent_variables[i]['header']:
                 name += ' [%s]' % self.independent_variables[i]['header']['units']
+            name = unicode(name).encode('ascii', 'replace')
             getattr(hist, self._hist_axes_getters[i])().SetTitle(name)
             if 'labels' in self.independent_variables[i]:
                 for ibin, label in enumerate(self.independent_variables[i]['labels']):
@@ -109,14 +110,7 @@ class THFRootClass(ObjectWrapper):
         xval_ordered = []
         for i in xrange(self.dim):
             xval_ordered.append([])
-            for j, x in enumerate(xval[i]):
-                if j == 0:
-                    x_highest = x
-                    xval_ordered[i].append(x)
-                else:
-                    if x > x_highest:
-                        x_highest = x
-                        xval_ordered[i].append(x)
+            xval_ordered[i] = sorted(xval[i])
 
         if 1 == self.dim:
             nbinsx = len(xval_ordered[0]) - 1
@@ -143,6 +137,7 @@ class THFRootClass(ObjectWrapper):
             name = self.independent_variables[i]['header']['name']
             if 'units' in self.independent_variables[i]['header']:
                 name += ' [%s]' % self.independent_variables[i]['header']['units']
+            name = unicode(name).encode('ascii', 'replace')
             getattr(hist, self._hist_axes_getters[i])().SetTitle(name)
             if 'labels' in self.independent_variables[i]: # set alphanumeric bin labels
                 for ibin, label in enumerate(self.independent_variables[i]['labels']):
@@ -152,6 +147,7 @@ class THFRootClass(ObjectWrapper):
             name = self.dependent_variable['header']['name']
             if 'units' in self.dependent_variable['header']:
                 name += ' [%s]' % self.dependent_variable['header']['units']
+            name = unicode(name).encode('ascii', 'replace')
             getattr(hist, self._hist_axes_getters[self.dim])().SetTitle(name)
 
         for i in xrange(len(self.xval[0])):
@@ -164,6 +160,7 @@ class THFRootClass(ObjectWrapper):
         error_hists = []
         error_labels = {}
         error_indices = {}
+        index = 0
 
         is_number_list = self.is_number_var(self.dependent_variable)
 
@@ -171,35 +168,20 @@ class THFRootClass(ObjectWrapper):
 
             if not is_number_list[i]: continue # skip non-numeric y values
 
-            # process the labels to ensure uniqueness
-            observed_error_labels = {}
+            # process the error labels to ensure uniqueness
+            ArrayWriter.process_error_labels(value)
+
             for error in value.get('errors', []):
-                label = error.get('label', '')
-
-                if label not in observed_error_labels:
-                    observed_error_labels[label] = 0
-                observed_error_labels[label] += 1
-
-                if observed_error_labels[label] > 1:
-                    error['label'] = label + '_' + str(observed_error_labels[label])
-
-                # append "_1" to first error label that has a duplicate
-                if observed_error_labels[label] == 2:
-                    for error1 in value.get('errors', []):
-                        error1_label = error1.get('label', 'error')
-                        if error1_label == label:
-                            error1['label'] = label + "_1"
-                            break
-
-            for index, error in enumerate(value.get('errors', []), 1):
                 if 'label' not in error:
                     error['label'] = 'error'
                 label = error['label']
+                if label not in error_labels:
+                    index += 1
+                    error_indices[index] = label
                 if 'symerror' in error and label not in error_labels:
                     error_labels[label] = 'symerror'
                 elif 'asymerror' in error and error_labels.get(label, 'symerror') == 'symerror':
                     error_labels[label] = 'asymerror'
-                error_indices[index] = label
 
         yvals = []
         for index in xrange(1, len(error_labels) + 1):
@@ -257,9 +239,13 @@ class THFRootClass(ObjectWrapper):
         for i in xrange(self.dim):
             xval.append([])
             i_var = self.independent_variables[i]['values']
-            for x in i_var:
-                xval[i].append(x['low'])
-            xval[i].append(i_var[-1]['high'])
+            for ix, x in enumerate(i_var):
+                if not is_number_list[ix] and 'labels' not in self.independent_variables[i]:
+                    continue  # skip defining bins for non-numeric y values unless alphanumeric bin labels are present
+                if x['low'] not in xval[i]:
+                    xval[i].append(x['low'])
+                if x['high'] not in xval[i]:
+                    xval[i].append(x['high'])
 
         try:
             hist = self._create_hist(xval)
@@ -297,8 +283,11 @@ class TGraph2DErrorsClass(ObjectWrapper):
     def create_objects(self):
         self.calculate_total_errors()
 
-        if self.xerr_plus[0] != self.xerr_minus[0] or self.xerr_plus[1] != self.xerr_minus[1] \
-                or self.yerr_plus != self.yerr_minus:
+        # check that errors are symmetric (within a tolerance to allow for numerical rounding)
+        tol = 1e-15
+        if any([a - b > tol for a, b in zip(self.xerr_plus[0], self.xerr_minus[0])]) or \
+           any([a - b > tol for a, b in zip(self.xerr_plus[1], self.xerr_minus[1])]) or \
+           any([a - b > tol for a, b in zip(self.yerr_plus, self.yerr_minus)]):
             return []
 
         if len(self.xval[0]):
@@ -317,12 +306,15 @@ class TGraph2DErrorsClass(ObjectWrapper):
         xname = self.independent_variables[0]['header']['name']
         if 'units' in self.independent_variables[0]['header']:
             xname += ' [%s]' % self.independent_variables[0]['header']['units']
+        xname = unicode(xname).encode('ascii', 'replace')
         yname = self.independent_variables[1]['header']['name']
         if 'units' in self.independent_variables[1]['header']:
             yname += ' [%s]' % self.independent_variables[1]['header']['units']
+        yname = unicode(yname).encode('ascii', 'replace')
         zname = self.dependent_variable['header']['name']
         if 'units' in self.dependent_variable['header']:
             zname += ' [%s]' % self.dependent_variable['header']['units']
+        zname = unicode(zname).encode('ascii', 'replace')
 
         graph.GetXaxis().SetTitle(xname)
         graph.GetYaxis().SetTitle(yname)
@@ -362,9 +354,11 @@ class TGraphAsymmErrorsRootClass(ObjectWrapper):
         xname = self.independent_variables[0]['header']['name']
         if 'units' in self.independent_variables[0]['header']:
             xname += ' [%s]' % self.independent_variables[0]['header']['units']
+        xname = unicode(xname).encode('ascii', 'replace')
         yname = self.dependent_variable['header']['name']
         if 'units' in self.dependent_variable['header']:
             yname += ' [%s]' % self.dependent_variable['header']['units']
+        yname = unicode(yname).encode('ascii', 'replace')
         graph.GetXaxis().SetTitle(xname)
         graph.GetYaxis().SetTitle(yname)
 
@@ -380,8 +374,8 @@ class ROOT(ArrayWriter):
         self.extension = 'root'
 
     def _write_table(self, data_out, table):
-        data_out.mkdir(table.name)
-        data_out.cd(table.name)
+        data_out.mkdir(table.name.replace('/','-').replace('$', '').replace('\\',''))
+        data_out.cd(table.name.replace('/','-').replace('$', '').replace('\\',''))
 
         # if no independent variables, use bins of unit width and centred on integers (1, 2, 3, etc.)
         if not table.independent_variables and table.dependent_variables:
@@ -415,16 +409,26 @@ class ROOT(ArrayWriter):
             graph.Write()
 
     def _prepare_outputs(self, data_out, outputs):
+        """ Open a ROOT file with option 'RECREATE' to create a new file (the file will
+        be overwritten if it already exists), and using the ZLIB compression algorithm
+        (with compression level 1) for better compatibility with older ROOT versions
+        (see https://root.cern.ch/doc/v614/release-notes.html#important-notice ).
+
+        :param data_out:
+        :param outputs:
+        :return:
+        """
+        compress = ROOTModule.ROOT.CompressionSettings(ROOTModule.ROOT.kZLIB, 1)
         if isinstance(data_out, (str, unicode)):
             self.file_emulation = True
-            outputs.append(ROOTModule.TFile.Open(data_out, 'RECREATE'))
+            outputs.append(ROOTModule.TFile.Open(data_out, 'RECREATE', '', compress))
         # multiple tables - require directory
         elif isinstance(data_out, ROOTModule.TFile):
             outputs.append(data_out)
         else:  # assume it's a file like object
             self.file_emulation = True
             filename = os.path.join(tempfile.mkdtemp(),'tmp.root')
-            outputs.append(ROOTModule.TFile.Open(filename,'RECREATE'))
+            outputs.append(ROOTModule.TFile.Open(filename, 'RECREATE', '', compress))
 
     def write(self, data_in, data_out, *args, **kwargs):
         """
